#!/usr/bin/env python3
# python3 OK

from astropy.io import fits
import numpy as np
import os
import argparse
import focasifu as fi
import sys
from multiprocessing import Pool, cpu_count, current_process
import time


def correlate_each(args):
    data = args[0]
    sky1d = args[1]
    ch_sky_reg = args[2]
    chstart = args[3]
    chend = args[4]

    #print(data.shape)
    #print(sky1d.shape)
        
    dy = np.zeros(data.shape[1:3], dtype=np.float32)
    for i in range(chstart, chend+1):
        if current_process().name == 'ForkPoolWorker-1':
            sys.stdout.write('\r\t\t {:.1f}% done'
                        .format(float(i-chstart)/float(chend - chstart + 1)*100.))
            sys.stdout.flush()            
        for j in range(data.shape[2]):
            #print(i, j, ch_sky_reg[i])
            dy[i,j] = fi.cross_correlate(data[:,i,j], sky1d[ch_sky_reg[i],:],
                                         sep=0.01, kind='cubic', half_win=1)
    sys.stdout.write('\r\t\t 100.0% done')
    sys.stdout.flush()
    return dy


def multi_process_correlate(data, ref1d, ch_sky_reg):
    print('\t Correlating')

    # Getting the number of CPU cores
    cpunum = cpu_count()
    print('\t\t Number of CPU: {}'.format(cpunum))

    # Making parameters
    chstep = int(data.shape[1]/cpunum)
    params = []
    for i in range(cpunum-1):
        params.append([data, ref1d, ch_sky_reg, i*chstep, (i+1)*chstep-1])
    params.append([data, ref1d, ch_sky_reg, chstep*(cpunum-1), data.shape[1]-1])

    # Multiprocessing
    p = Pool(cpunum)
    start = time.time()
    results = p.map(correlate_each, params)
    elapsed_time = time.time() - start
    print('\n\t\t Elapsed time: {:.1f} sec'.format(elapsed_time))

    # Merging data
    results_np = np.array(results)
    dy = np.zeros(data.shape[1:3], dtype=np.float32)
    for i in range(results_np.shape[0]):
        dy = dy + results_np[i]

    return dy


def get_sky_shift(comp, scale=1.0, overwrite=False, ql=False):
    print('\n#############################')
    print('Getting deviations between object spectra and sky specum.')
    hdl = fits.open(comp)
    comp_data = hdl[0].data
    basename = hdl[0].header['FRAMEID']
    outfile = basename + '.sky_shift.dat'
    
    if not fi.check_version(hdl):
        return outfile, False
    
    if os.path.isfile(outfile):
        if not overwrite:
            print('\t Sky shift data file already exits. '+outfile)
            print('\t This procedure is skipped.')
            return outfile, True

    if not ql:
        ## Sky regions
        sky_reg = np.array([[10, comp_data.shape[2]-2],
                   [2, comp_data.shape[2]-2],
                   [2, 55]])

        ## correspondence between Ch and sky_region
        ch_sky_reg = [1, 2, 2, 2, 1,
                      1, 1, 1, 1, 0,
                      0, 0, 0, 0, 1,
                      1, 1, 1, 1, 1,
                      2, 2, 2, 1]

        # Making 1D sky spectrum of the comparison data.
        comp_sky1d = np.zeros((len(sky_reg), comp_data.shape[0]))
        for i in range(len(sky_reg)):
            temp =np.mean(comp_data[:, 23, sky_reg[i,0]:sky_reg[i,1]], axis=1)
            # Because scipy.ndimage.interpolation.shift can not deal with NaN,
            # NaN is replaced to zero.
            temp2 = np.nan_to_num(temp)
            comp_sky1d[i] = temp2*scale

        # Deriving shifts of the sky spectrum
        dy = multi_process_correlate(comp_data, comp_sky1d, ch_sky_reg)
    else:
        dy = np.zeros(comp_data.shape[1:3], dtype=np.float32)

    np.savetxt(outfile, dy, fmt='%.2f')

    return outfile, True
    
if __name__ == '__main__':
    parser=argparse.ArgumentParser(description='This is the '+\
            'script for deriving deviations between object spectra ' + \
                                   'and sky one.')
    parser.add_argument('-o', help='Overwrite flag', dest='overwrite',\
            action='store_true',default=False)
    parser.add_argument('comp', help='Comparison data cube.')
    parser.add_argument('-ql', help='Quick look mode', dest='ql',
                        action='store_true',default=False)
    args = parser.parse_args()

    get_sky_shift(args.comp, overwrite=args.overwrite, ql=args.ql)
