#!/usr/bin/env python
# python3 ok

import matplotlib.pyplot as plt
from matplotlib.colors import LogNorm
import re
import numpy as np
from astropy.io import fits
import argparse
import copy
import os
import sys

import focasifu as fi

# Not to display warning.
temp_stderr = sys.stderr
sys.stderr = open('/dev/null', 'w')
from identify_dispersion import nsum
sys.stderr = temp_stderr  # Back to the stadard error output

def ReadIDFile(idfname):
    # idfname: ID file name
    # w: half window size for integrating intensity
    #
    # coo_np:
    # numpy array for emission line coordinates and intensities.
    # Each element has the following infomation: (x, y) in the
    # Python coordinate (start from 0, not 1).

    coo = []
    
    # Reading ID-file
    f = open(idfname)
    lines = f.readlines()
    f.close()
    
    x = -1
    for i in range(len(lines)):
        fields = re.split('[,\[\t\n]',lines[i])
        if fields[0] == 'begin':
            x0 = x
            x = int(fields[2]) - 1
            if x == x0:
                del coo[len(coo)-n:len(coo)]
        elif fields[1] == 'features':
            n = int(fields[2])
            for j in range(n):
                i += 1
                fields = lines[i].split()
                y = float(fields[0]) - 1.0
                w1 = float(fields[1])
                w2 = float(fields[2])
                par1 = float(fields[3])
                par2 = int(fields[4])
                par3 = int(fields[5])
                coo.append([x, y, w1, w2, par1, par2, par3])
        else:
            i += 1    
                
    return  coo

def RemoveBadData(coo, intensity, fac=0.5):
    # Remove data points from "coo" array if their intensies are
    # lower than those of the neighboring points.
    
    i = 0
    while coo[i][0] == coo[0][0]:
        inte = intensity[i]
        w0 = coo[i][3]
        k=0
        flg = 0
        while k != len(coo):
            if coo[k][3] == w0:
                if coo[k][0] > coo[i][0] and flg == 0:
                    inte = intensity[i]
                    flg = 1
                   
                if intensity[k] < fac * inte:
                    coo.pop(k)
                    intensity.pop(k)
                    k-=1
                else:
                    inte = intensity[k]
            k+=1
        i+=1
    return


def GetIntensity(rootname, coo, wx=int(nsum/2), wy=1):
    # Getting image data
    data = fits.getdata('../' + rootname + '.fits')

    # Getting intesity by integrating from x-wx to x+wx and
    # from y-wy to y+wy.
    intensity = []
    for i in range(len(coo)):
        xi = coo[i][0]
        yi = int(coo[i][1]+0.5)
        # In REIDENTIFY/IRAF, when x=50 and nsum=10, the integration range
        # is from 46 to 55 in the IRAF coordinate.
        # print(xi-int(nsum/2)+1, xi+int(nsum/2)+2)
        intensity.append(np.sum(data[yi-wy:yi+wy+1, xi-wx+1:xi+wx+2]))

    return intensity


def WriteIDFile(rootname, coo, overwrite=False):

    idfname = 'id' + rootname
    orgfname = idfname+'.org'
    if os.path.isfile(idfname):
        print('\n\t There is ' + idfname + '.')
        if not os.path.isfile(orgfname):
            print('\t The file is renamed to ' + orgfname + '.')
            os.rename(idfname, orgfname)
        else:
            print('\t There is also ' + orgfname + '.')
            if overwrite:
                print('\t '+ idfname + ' is overwritten.')
            else:
                print('\t This script is skipped.')
                return
            
    newlines = []
    i = 0
    while i < len(coo) -1 :
        newlines.append('# Created by rmbaddata.py\n')
        newlines.append('begin\tidentify '+rootname+'['+
                        str(coo[i][0]+1)+',*]\n')
        newlines.append('\tid\t'+rootname+'\n')
        newlines.append('\ttask\tidentify\n')
        newlines.append('\timage\t'+rootname+'['+
                        str(coo[i][0]+1)+',*]\n')
        newlines.append('\tunits\tAngstroms\n')
                        
        i0 = i
        while coo[i][0] == coo[i0][0] and i < len(coo) - 1:
            i+=1

        newlines.append('\tfeatures\t'+str(i-i0)+'\n')

        for k in range(i0,i):
            newlines.append('\t{:14.2f} {:11f} {:11f} {:5.1f} {:1d} {:1d}\n'.format(
                coo[k][1] + 1.0, coo[k][2], coo[k][3], coo[k][4], coo[k][5],
                coo[k][6]))

    f = open(idfname, "w")
    f.writelines(newlines)
    f.close()
                            
    return

def PlotPosition(rootname, coo1, coo2):
    data = fits.getdata('../' + rootname + '.fits')
    
    plt.imshow(data, origin='bottom', norm=LogNorm(vmin=1,vmax=np.max(data)), \
               aspect='auto')
    coo1_np = np.array(coo1)
    coo2_np = np.array(coo2)
    plt.scatter(coo1_np[:,0], coo1_np[:,1], s=10, c='red')
    plt.scatter(coo2_np[:,0], coo2_np[:,1], s=10, c='blue')
    plt.xlabel('X (pix)')
    plt.ylabel('Y (pix)')
    plt.title(rootname)
    plt.show()
    return
    

def rmidpoints(basenames, fac=0.5, database='database', overwrite=False):
    print('\n#############################')
    print('Removing bad points identified by REIDENTIFY task.')
    
    # entering the channel image directory.
    print('\t Entering the channel image directory, \"' + fi.chimagedir +
          '/' +database)
    os.chdir(fi.chimagedir + '/' + database)

    for basename in basenames:
        for i in range(1,25):
            rootname = basename + '.ch{:02d}'.format(i)
            idfname = 'id' + rootname
            coo = ReadIDFile(idfname)
            coo_org = copy.copy(coo)
            intensity = GetIntensity(rootname, coo)
            RemoveBadData(coo, intensity, fac=fac)
            PlotPosition(rootname, coo_org, coo)
            WriteIDFile(rootname, coo, overwrite=overwrite)

    os.chdir('../..')
    return

if __name__ == '__main__':
    parser=argparse.ArgumentParser(description= \
            'This is the script for checking the identified positions of '+
            'the comparison lines.')
    parser.add_argument('ifnames', help='Comma-separated input basenames')
    parser.add_argument('-fac', help='Threshold factor.', type=float,
                        default=0.5)
    parser.add_argument('-o', help='Overwrite flag', dest='overwrite',
                        action='store_true',default=False)
    args = parser.parse_args()

    rmidpoints(args.ifnames.split(','), args.fac, overwrite=args.overwrite)
