#!/usr/bin/env python3
# python3 OK

from astropy.io import fits
import numpy as np
import os
import argparse
import focasifu as fi
from scipy.ndimage.interpolation import shift as imshift
import matplotlib.pyplot as plt


def skysub(infile, comp, scale=1.0, overwrite=False):
    print('\n#############################')
    print('Sky subtraction')
    hdl = fits.open(infile)
    hdr = hdl[0].header
    basename = hdr['FRAMEID']
    outfile = basename + '.ss.fits'
    
    if not fi.check_version(hdl):
        return outfile, False
    
    if os.path.isfile(outfile):
        if not overwrite:
            print('\t Sky subtracted file already exits. '+outfile)
            print('\t This procedure is skipped.')
            return outfile, True
        
    scidata = hdl[0].data

    dy = np.loadtxt(comp+'.sky_shift.dat')

    ## Sky regions
    sky_reg = np.array([[10, scidata.shape[2]-2],  # 10:62 = 52pix
               [2, scidata.shape[2]-2],  # 2:62 = 60pix
               [2, 55]])  # 2:55 = 53pix

    ## correspondence between Ch and sky_region
    ch_sky_reg = [1, 2, 2, 2, 1,
                  1, 1, 1, 1, 0,
                  0, 0, 0, 0, 1,
                  1, 1, 1, 1, 1,
                  2, 2, 2, 1]
                  
    # Maiking 1D sky spectrum
    sky1d = np.zeros((len(sky_reg), scidata.shape[0]))
    for i in range(len(sky_reg)):
        temp =np.mean(scidata[:, 23, sky_reg[i,0]:sky_reg[i,1]], axis=1)
        # Because scipy.ndimage.interpolation.shift can not deal with NaN,
        # NaN is replaced to zero.
        temp2 = np.nan_to_num(temp)
        sky1d[i] = temp2*scale

    # Plotting sky spectrum.
    crpix = hdr['CRPIX3']
    crval = hdr['CRVAL3']
    cdelt = hdr['CD3_3']

    npix = sky1d.shape[1]
    start = crval - (crpix-1)*cdelt
    stop = crval + (npix - crpix + 0.5)*cdelt
    lam = np.arange(start ,stop, cdelt)

    for i in range(3):
        plt.plot(lam, sky1d[i,:])
    plt.title('Sky spectrum')
    plt.xlabel('Lambda (Angstrom)')
    plt.ylabel('Count')
    plt.show()
    
    # Subtracting sky spectrum.
    print('\t Subtracting the sky specturm.')

    outdata = np.zeros(scidata.shape, dtype=np.float32)
    for i in range(scidata.shape[1]):
        for j in range(scidata.shape[2]):
            sky1d_shifted = imshift(sky1d[ch_sky_reg[i],:], dy[i,j],
                                    order=5, mode='nearest')
            outdata[:,i,j] = scidata[:,i,j] - sky1d_shifted * scale

    # Writing the output file
    outhdu = fits.PrimaryHDU(outdata)
    outhdl = fits.HDUList([outhdu])
    outhdl[0].header = hdr
    outhdl[0].header['SKY_SCL'] = (scale, 'Scale factor for sky spectrum')
    outhdl.writeto(outfile, overwrite=overwrite)
    outhdl.close()
    hdl.close()
    print('\t Sky subtracted file: '+outfile)
    return outfile, True


if __name__ == '__main__':
    parser=argparse.ArgumentParser(description='This is the '+\
            'script for subtracting the sky background spectrum.')
    parser.add_argument('-o', help='Overwrite flag', dest='overwrite',\
            action='store_true',default=False)
    parser.add_argument('infile', help='Input FITS file.')
    parser.add_argument('comp', help='Comparison ID.')
    parser.add_argument('-scale', help='Scale factor applied for '+\
            'the sky spectrum. (default: 1.0)', default=1.0, type=float)
    args = parser.parse_args()

    skysub(args.infile, args.comp, scale=args.scale, overwrite=args.overwrite)
